"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.errorHandler = exports.body = exports.query = void 0;

var _schm = _interopRequireDefault(require("schm"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var convertEmptyToTrue = function convertEmptyToTrue(object) {
  return Object.keys(object).reduce(function (finalObject, key) {
    return _objectSpread({}, finalObject, _defineProperty({}, key, object[key] === "" ? true : object[key]));
  }, {});
};
/**
 * Returns an express middleware that validates and parses querystring based
 * on a given schema.
 * @example
 * const express = require('express')
 * const { query } = require('schm-express')
 *
 * const app = express()
 *
 * // request /?foo&bar=1&bar=baz
 * app.get('/', query({ foo: Boolean, bar: [String] }), (req, res) => {
 *   console.log(req.query) // { foo: true, bar: ['1', 'baz'] }
 * })
 */


var query = function query(params) {
  return function (req, res, next) {
    // $FlowFixMe
    var querySchema = (0, _schm.default)(params);
    var values = convertEmptyToTrue(req.query);
    querySchema.validate(values).then(function (parsed) {
      req.query = parsed;
      next();
    }).catch(function (errors) {
      req.schmError = true;
      next(errors);
    });
  };
};
/**
 * Returns an express middleware that validates and parses request body based
 * on a given schema.
 * @example
 * const express = require('express')
 * const bodyParser = require('body-parser')
 * const { body } = require('schm-express')
 *
 * const app = express()
 * app.use(bodyParser.json())
 * app.use(bodyParser.urlencoded({ extended: false }))
 *
 * // send { foo: 1, bar: 'baz' }
 * app.post('/', body({ foo: Boolean, bar: [String] }), (req, res) => {
 *   console.log(req.body) // { foo: true, bar: ['baz'] }
 * })
 */


exports.query = query;

var body = function body(params) {
  return function (req, res, next) {
    // $FlowFixMe
    var bodySchema = (0, _schm.default)(params);
    bodySchema.validate(req.body).then(function (parsed) {
      req.body = parsed;
      next();
    }).catch(function (errors) {
      req.schmError = true;
      next(errors);
    });
  };
};
/**
 * Handles errors from schm-express.
 * @example
 * const express = require('express')
 * const { query } = require('schm-express')
 *
 * const app = express()
 *
 * // request / without querystring
 * app.post('/', query({ foo: { type: Boolean, required: true } }), (req, res) => {
 *   ...
 * })
 *
 * app.use(errorHandler())
 *
 * // it will respond with 400 and error descriptor in response body
 */


exports.body = body;

var errorHandler = function errorHandler() {
  return function (err, req, res, next) {
    if (req.schmError) {
      return res.status(400).json(err);
    }

    return next(err);
  };
};

exports.errorHandler = errorHandler;